@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # vim: fileencoding=utf-8 expandtab ts=4 nospell
 
-# SPDX-FileCopyrightText: 2020 Benedict Harcourt <ben.harcourt@harcourtprogramming.co.uk>
+# SPDX-FileCopyrightText: 2020-2021 Benedict Harcourt <ben.harcourt@harcourtprogramming.co.uk>
 #
 # SPDX-License-Identifier: BSD-2-Clause
 
@@ -18,8 +18,8 @@ Find this project at:
 
 from __future__ import annotations
 
-from .table import Table, ModelWrapper as TableModel, unique
+from .table import Table, ModelWrapper as TableModel, subtable, unique
 from .join import JoinTable, JoinWrapper as JoinModel
 
 
-__all__ = ["Table", "TableModel", "JoinTable", "JoinModel", "unique"]
+__all__ = ["Table", "TableModel", "JoinTable", "JoinModel", "subtable", "unique"]
